////////////////////////////////////////////////////////////////
//
// Copyright (C) 2005 Affymetrix, Inc.
//
// This program is free software; you can redistribute it and/or modify 
// it under the terms of the GNU General Public License (version 2) as 
// published by the Free Software Foundation.
// 
// This program is distributed in the hope that it will be useful, 
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
// General Public License for more details.
// 
// You should have received a copy of the GNU General Public License 
// along with this program;if not, write to the 
// 
// Free Software Foundation, Inc., 
// 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
////////////////////////////////////////////////////////////////

/**
 * @file   ConvertTest.cpp
 * @author Chuck Sugnet
 * @date   Wed May  4 15:45:33 2005
 * 
 * @brief  Testing the convert functions.
 * 
 */
#ifndef CONVERTTEST_H
#define CONVERTTEST_H

#include "Convert.h"
#include <string>
#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/CompilerOutputter.h>
#include <cppunit/extensions/TestFactoryRegistry.h>
#include <cppunit/ui/text/TestRunner.h>

using namespace std;
/**
 * @class ConvertTest
 * @brief cppunit class for testing conversion functions.
 */
class ConvertTest : public CppUnit::TestFixture
{
  CPPUNIT_TEST_SUITE( ConvertTest );
  CPPUNIT_TEST( testInteger );
  CPPUNIT_TEST( testUnsignedInteger );
  CPPUNIT_TEST( testFloat );
  CPPUNIT_TEST( testBool );
  CPPUNIT_TEST( testToString );
  CPPUNIT_TEST( testToStr );
  CPPUNIT_TEST_SUITE_END();

public:
  /** Test converting integers. */
  void testInteger();
  /** Test converting unsigned integers. */
  void testUnsignedInteger();
  /** Test converting floats. */
  void testFloat();
  /** Test converting booleans. */
  void testBool();
  /** Test for converting things to strings. */
  void testToString();
  /** Test for converting strings template. */
  void testToStr();
};

// Registers the fixture into the 'registry'
CPPUNIT_TEST_SUITE_REGISTRATION( ConvertTest );

/**
 * Make sure that toString functions work.
 */
void ConvertTest::testToString() {
  CPPUNIT_ASSERT( Convert::toString(10) == "10" );
  CPPUNIT_ASSERT( Convert::toString(-10) == "-10" );
  CPPUNIT_ASSERT( Convert::toString(+10) == "10" );

  CPPUNIT_ASSERT( Convert::toString(10.5) == "10.5" );
  CPPUNIT_ASSERT( Convert::toString(-10.5) == "-10.5" );
  CPPUNIT_ASSERT( Convert::toString(+10.5) == "10.5" );
}
  
/** 
 * Make sure that the integer conversion works.
 */
void ConvertTest::testInteger() {
  int i = 0;
  bool success = 0;
  i = Convert::toIntCheck("1", &success);
  CPPUNIT_ASSERT( i == 1 && success == true);

  i = Convert::toIntCheck("Q", &success);
  CPPUNIT_ASSERT( i == 0 && success == false);

  i = Convert::toIntCheck("Q", &success);
  CPPUNIT_ASSERT( i == 0 && success == false);

  i = Convert::toIntCheck("-12", &success);
  CPPUNIT_ASSERT( i == -12 && success == true);

  i = Convert::toIntCheck("+12", &success);
  CPPUNIT_ASSERT( i == 12 && success == true);
}

/** 
 * Make sure that the integer conversion works.
 */
void ConvertTest::testUnsignedInteger() {
  unsigned int i = 0;
  bool success = 0;
  string s = ToStr(UINT_MAX);
  i = Convert::toUnsignedIntCheck("1", &success);
  CPPUNIT_ASSERT( i == 1 && success == true);

  i = Convert::toUnsignedIntCheck("Q", &success);
  CPPUNIT_ASSERT( i == 0 && success == false);

  i = Convert::toUnsignedIntCheck(s.c_str(), &success);
  CPPUNIT_ASSERT( i == 0 && success == false);

  i = Convert::toUnsignedIntCheck("+12", &success);
  CPPUNIT_ASSERT( i == 12 && success == true);
}


/**
 * Make sure that float conversions work when they should.
 * Currently not testing for exact equality would need to 
 * write a "close enough" comparitor.
 */
void ConvertTest::testFloat() {
  float f = 0;
  bool success = true;
  f = Convert::toFloatCheck("1.0", &success);
  CPPUNIT_ASSERT( Convert::doubleCloseEnough(f, 1.0) && success == true);

  f = Convert::toFloatCheck("-1.0", &success);
  CPPUNIT_ASSERT( Convert::doubleCloseEnough(f, -1.0) && success == true);

  f = Convert::toFloatCheck("-1.G", &success);
  CPPUNIT_ASSERT( Convert::doubleCloseEnough(f, 0) && success == false);
}

/** 
 * Make sure that we understand bools and fail on junk.
 */
void ConvertTest::testBool() {
  bool success = true, value = true;

  // T!
  value = Convert::toBoolCheck("true", &success);
  CPPUNIT_ASSERT( value && success);
  value = Convert::toBoolCheck("TRUE", &success);
  CPPUNIT_ASSERT( value && success);
  value = Convert::toBoolCheck("1", &success);
  CPPUNIT_ASSERT( value && success);

  // F!
  value = Convert::toBoolCheck("false", &success);
  CPPUNIT_ASSERT( !value && success);
  value = Convert::toBoolCheck("FALSE", &success);
  CPPUNIT_ASSERT( !value && success);
  value = Convert::toBoolCheck("0", &success);
  CPPUNIT_ASSERT( !value && success);

  // bad!
  value = Convert::toBoolCheck("junk", &success);
  CPPUNIT_ASSERT( !value && !success);
  value = Convert::toBoolCheck("", &success);
  CPPUNIT_ASSERT( !value && !success);
}

/** Test for converting strings template. */
void ConvertTest::testToStr() {
  CPPUNIT_ASSERT( string("-5.295") == ToStr(-5.295) );
  CPPUNIT_ASSERT( string("100") == ToStr(100) );
  CPPUNIT_ASSERT( string("Some stupid string") == ToStr("Some stupid string") );
  CPPUNIT_ASSERT( string("Cast to string") == ToStr(string("Cast to string")) );
}

#endif /* CONVERTTEST_H */
